/* YourDuino.com Example: Multiple DS18B20 Temperature Sensors
   Displayed on 4x20 character LCD display

   DS18B20 Pinout (Left to Right, pins down, flat side toward you)
  - Left   = Ground
  - Center = Signal (Pin 9):  (with 3.3K to 4.7K resistor to +5 or 3.3 )
  - Right  = +5 or +3.3 V

   terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <OneWire.h>// Get 1-wire Library here: http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <DallasTemperature.h>//Get DallasTemperature Library here:  http://milesburton.com/Main_Page?title=Dallas_Temperature_Control_Library
#include <Wire.h>// Wire (I2C) Library
#include <PID_v1.h> //PID LIB for fan speed
#include <LiquidCrystal.h>// LCD Library
#include <LiquidCrystal_I2C.h>
#include <avr/wdt.h>//Watchdog Lib
// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol

#define ONE_WIRE_BUS 4    // Data wire is plugged into port 4 on the Arduino 
#define pwmH 6
#define pwmTR 5



/*-----( Declare objects )-----*/
// Setup a oneWire instance to communicate with any OneWire devices
// (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);

// Pass address of our oneWire instance to Dallas Temperature.
DallasTemperature sensors(&oneWire);


//-----------------------------CHANGE THESE AS REQUIRED---------------------------------------------------------------------------------

// Start the LCD display library
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address change this 0x3F or 0x27 if nothing on the LCD
double HSsetpoint = 40;      //Heatsink Target Temperature
double TRsetpoint = 50;      //Toroid Target Temperature
int MinVolts = 47;           //Battery Volts to shut down
int TimeAtMinVolts = 20;    //How many seconds below MinVolts till shout down is triggered
int RestartTime = 10;        // How many minutes to restart if battery volts recover above MinVolts
//DeviceAddress Probe01 = { 0x28, 0xEE, 0xDC, 0x17, 0x2E, 0x16, 0x01, 0x9D }; // Address printed on Sensor connected to Heatsink 8KW
//DeviceAddress Probe02 = { 0x28, 0xEE, 0x2E, 0xE1, 0x1C, 0x16, 0x01, 0x51 }; // Address printed on Sensor connected to Toroid
DeviceAddress Probe01 = { 0x28, 0xEE, 0xDC, 0x17, 0x2E, 0x16, 0x01, 0x9D }; // Address printed on Sensor connected to Heatsink
DeviceAddress Probe02 = { 0x28, 0xEE, 0x2E, 0xE1, 0x1C, 0x16, 0x01, 0x51 }; // Address printed on Sensor connected to Toroid
//------------------------------STOP HERE IF YOU ARE NOT SURE---------------------------------------------------------------------------


int HSmax;
int TRmax;
int FANtempH;
int FANtempT;
double tempTR;
int tempTR1;
double tempHS;
int tempHS1;
double fanspeedH;
double fanspeedT;
double fanpwmH;
double fanpwmT;
int fanpercentH;
int fanpercentT;

const int numReadings = 10;
double Setpoint, Input, Output;

int readings;      // the readings from the analog input
int readIndex = 0;              // the index of the current reading
int total = 0;                  // the running total
int average = 0;                // the average

int voltsinputPin = A7;

boolean singlecase = false; // true for all in one case, false for divided case with seperate fans for heatsink and toriod.
float R1 = 100000.0;            // resistance of R1 (100K)
float R2 = 10000.0;              // resistance of R2 (8K2) For setting up voltage divider to get correct voltage reading Use in conjunction with trim pot on pcb
double vin = 0.0;
double vout = 0.0;
int beeper =  11;// the number of the beeper pin
int beeperstate = LOW;
int ShutDown =  3;// the number of the shutdown pin
boolean beeping = false;
unsigned long previousMillis = 0;
const long interval = 750;           // interval at which to beep (milliseconds)
long LVStart;
long timenow;
long vingood;

double Kp = 120, Ki = 20, Kd = 3; //variables for setting PID for fan speed control
PID HSPID(&tempHS, &fanpwmH, &HSsetpoint, Kp, Ki, Kd, REVERSE);
PID TRPID(&tempTR, &fanpwmT, &TRsetpoint, Kp, Ki, Kd, REVERSE);

void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600);
  wdt_enable(WDTO_1S); // Setup watchdog to reset after 1 second if no activity
  //------- Initialize the Temperature measurement library--------------
  sensors.begin();
  // set the resolution to 10 bit (Can be 9 to 12 bits .. lower is faster)
  sensors.setResolution(Probe01, 10);
  sensors.setResolution(Probe02, 10);


  //---------------- Initialize the lcd ------------------
  lcd.begin(20, 4);        // initialize the lcd for 20 chars 4 lines, turn on backlight
  lcd.setCursor(0, 0);
  HSPID.SetMode(AUTOMATIC);
  TRPID.SetMode(AUTOMATIC);

  pinMode(beeper, OUTPUT);
  //  pinMode(ShutDown, OUTPUT);
  // initialize all the readings to 0:
  for (int thisReading = 0; thisReading < numReadings; thisReading++) {
    readings = 0;
  }

}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  wdt_reset();   // Reset watchdog timer



  lcd.home();
  lcd.backlight();  //Backlight ON if under program control
  lcd.setCursor(0, 3);
  lcd.print("V :");
  lcd.print(vin);
  sensors.requestTemperatures(); // Send the command to get temperatures
  lcd.setCursor(0, 2);
  lcd.print("                      ");
  timenow = millis();
  lcd.setCursor(0, 0); //Start at character 0 on line 0
  lcd.print("H:");
  {

    tempHS = sensors.getTempC(Probe01);


    if (tempHS == -127.00) // Measurement failed or no device found
    {
      lcd.print("Error");
      tempHS = tempHS1;
    }
    else
    {
      lcd.print(tempHS, 0);
      tempHS1 = tempHS;
      HSmax = max (HSmax, tempHS);
      lcd.print(" M:");
      lcd.print(HSmax);
    }

    HSPID.Compute();

    {

      tempTR = sensors.getTempC(Probe02);

      if (tempTR == -127.00) // Measurement failed or no device found
      {
        lcd.print("Error");
        tempTR = tempTR1;
      }
      else
      {
        lcd.print(" T:");
        lcd.print(tempTR, 0);
        tempTR1 = tempTR;
        TRmax = max (TRmax, tempTR);
        lcd.print(" M:");
        lcd.print(TRmax);
      }
    }


    TRPID.Compute();
  }

  analogWrite (pwmTR, fanpwmT);
  analogWrite (pwmH, fanpwmH);
  lcd.setCursor(0, 1);
  lcd.print("HSFAN%    ");
  lcd.setCursor(6, 1);
  lcd.print(fanpwmH / 2.55, 0);
  lcd.setCursor(9, 1);
  lcd.print(" TRFAN%    ");
  lcd.setCursor(16, 1);
  lcd.print(fanpwmT / 2.55, 0);


  {
    // subtract the last reading:
    total = total - readings;
    // read from the sensor:
    readings = analogRead(voltsinputPin);
    // add the reading to the total:
    total = total + readings;
    // advance to the next position in the array:
    readIndex = readIndex + 1;

    // if we're at the end of the array...
    if (readIndex >= numReadings) {
      // ...wrap around to the beginning:
      readIndex = 0;
    }

    // calculate the average:
    average = total / numReadings;

    vout = (average);
    vin = vout / 39;    // Voltage divider calculation
    if ((vin >= MinVolts) && (vingood == 0))
    { vingood = millis();
    }


    if ((vingood > 0) && ((timenow - vingood) >  (RestartTime * 60000)))
    { vingood = 0;
    }

    if ((vin >= MinVolts) && (vingood == 0))
    {
      lcd.setCursor(9, 3);
      lcd.print("VOLTS GOOD");
      digitalWrite (ShutDown, LOW);
      beeping = false;
    }
    if (vin >= MinVolts)
    { LVStart = 0;
    }

    else if (LVStart == 0)
    {
      LVStart = millis();
    }

    if ((LVStart > 0) && (timenow - LVStart >= (TimeAtMinVolts * 1000)))
    { lcd.setCursor(9, 3);
      lcd.print("LOW VOLTS ");
      digitalWrite (ShutDown, HIGH);
      beeping = true;


    }
    if (beeping == true)
    {
      unsigned long currentMillis = millis();

      if (currentMillis - previousMillis >= interval) {
        // save the last time you blinked the LED
        previousMillis = currentMillis;

        // if the LED is off turn it on and vice-versa:
        if (beeperstate == LOW) {
          beeperstate = HIGH;
        } else {
          beeperstate = LOW;
        }
      }




    }
  }
  if (beeping == false) {
    beeperstate = LOW;
  }
  // set the LED with the ledState of the variable:
  digitalWrite(beeper, beeperstate);


}

//--(end main loop )---






